/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: semaphore.cpp,v 1.4 2003/06/23 09:27:19 robert Exp $

----------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "semaphore.h"


Semaphore::Semaphore(const char *name)
{
    this->count = 1;
    pthread_cond_init(&semCond, NULL);
    pthread_mutex_init(&semMutex, NULL);
    if (name)
        this->name = strdup(name);
    else 
        this->name = NULL;
}

Semaphore::~Semaphore(void)
{
    pthread_mutex_destroy(&semMutex);
    pthread_cond_destroy(&semCond);
    if (name)
        free(name);
}

void Semaphore::signal(void)
{
    pthread_mutex_lock(&semMutex);
    count++;
    //printf("signal %s: %d\n", name, count);
    pthread_mutex_unlock(&semMutex);
    pthread_cond_signal(&semCond);
}

bool Semaphore::wait(void)
{
    pthread_mutex_lock(&semMutex);
    count--;
    //printf("wait %s: %d\n", name, count);
    while(count <= 0)
        pthread_cond_wait(&semCond, &semMutex);

    pthread_mutex_unlock(&semMutex);

    return true;
}
