/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: write.h,v 1.8 2003/06/23 09:27:19 robert Exp $

----------------------------------------------------------------------------*/
#ifndef FILERENAME_H
#define FILERENAME_H

#include "thread.h"
#include "semaphore.h"
#include "trm.h"
#include "track.h"
#include "context.h"

class TunePimp;
class FileCache;

class WriteThread : public Thread
{
    public:

                 WriteThread(TunePimp     *tunePimp,
                              FileCache    *cache);
        virtual ~WriteThread(void);
        void     wake       (void);
        void     threadMain (void);

    private:

        bool     writeTrack     (Track *track, const Metadata &data);
        bool     createPath     (const string &path);
        void     cleanPath      (const string &pathArg);
        void     makeNewFileName(const Metadata &data,
                                 string         &fileName,
                                 int             index);
        string   extractFileExt (const string &path);
        string   extractFileName(const string &path);
        string   extractFilePath(const string &file);
        string   extractVolume  (const string &file);

        bool          renameCheck  (Track *track, const Metadata &data);
        const string  sanitize     (const string &str);
        unsigned long fileOpenTest (const string &fileName);
        bool          diskSpaceTest(const string &fileName, 
                                    unsigned long fileSize);

        TunePimp       *tunePimp;
        FileCache      *cache;
        bool            exitThread;
        Semaphore      *sem;
};

#endif
